# Generated by Django 2.2.10 on 2020-02-10 18:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('entity_emailer', '0002_auto_20170919_1653'),
    ]

    operations = [
        migrations.AddField(
            model_name='email',
            name='exception',
            field=models.TextField(default=None, null=True),
        ),
    ]
